#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Tab Session Management extension for Epiphany written in Python
#
# Version: 0.2
#
# Copyright (C) 2006 George Notaras <gnotar@gmail.com>
#
# Save and load tab sessions at will.
#
# This plugin is UNOFFICIAL and completely UNSUPPORTED. It is released without
# any warranties of any kind.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the:
#
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston,
# MA 02111-1307  USA
#


from os.path import exists
import epiphany
import gtk

try:	# Python version 2.5 or newer
	import xml.etree.cElementTree as ET
except ImportError:
	try:	# Python version < 2.5
		import cElementTree as ET
	except ImportError:
		import elementtree.ElementTree as ET


# Tab Session Files' extension
TSFext = '.ephytabs'

# Menu Items
_ui_str = """
<ui>
 <menubar name="menubar">
  <menu name="ToolsMenu" action="Tools">
   <separator/>
   <menuitem name="SaveTabSession" action="SaveTabSession"/>
   <menuitem name="LoadTabSession" action="LoadTabSession"/>
   <separator/>
  </menu>
 </menubar>
</ui>
"""

# These are the callbacks: what our menu entries will do

def _show_dialog( window, what=None ):
	
	if what == 'SAVE':
		dialog = gtk.FileChooserDialog(
			title='Select a file to save the tab session',
			parent=None,
			action=gtk.FILE_CHOOSER_ACTION_SAVE,
			buttons=( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK),
			backend=None
			)
	elif what == 'LOAD':
		dialog = gtk.FileChooserDialog(
			title='Select an Epiphany tab session file',
			parent=None,
			action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK),
			backend=None
			)
	
	dialog.set_default_response( gtk.RESPONSE_OK )
	
	extfilter = gtk.FileFilter()
	extfilter.set_name('All files')
	extfilter.add_pattern('*')
	dialog.add_filter( extfilter )
	
	extfilter = gtk.FileFilter()
	extfilter.set_name('Epiphany Tab Sessions')
	extfilter.add_pattern( '*' + TSFext )
	dialog.add_filter( extfilter )
	
	dialog.set_filter( extfilter )
	
	SESfile = ''
	dialog.set_filename( SESfile )
	
	while True:
		response = dialog.run()
		
		if response != gtk.RESPONSE_OK:
			SESfile = None
			break
		else:
			SESfile = dialog.get_filename()
			
			if what == 'SAVE':
				if exists( SESfile ):
					# Show warning dialog
					warn_dlg = gtk.MessageDialog(
								parent=dialog,
								flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
								type=gtk.MESSAGE_WARNING,
								buttons=(gtk.BUTTONS_YES_NO),
								message_format='There is a file with that name already.\nWould you like to overwrite it?'
								)
								
					result = warn_dlg.run()
					warn_dlg.destroy()
					
					# Overwrite the file
					if result == gtk.RESPONSE_YES:
						break
				else:
					break
			elif what == 'LOAD':
				break
	
	dialog.destroy()
	
	return SESfile



def _save_tab_session( action, window ):
	
	# Ask for File
	SESfile = _show_dialog( window, 'SAVE' )
	
	# Returns None if dialog canceled
	if not SESfile:
		return
	else:
		if not SESfile.endswith( TSFext ):
			SESfile += TSFext
	
	SESroot = ET.Element( 'session' )
	SESroot.text = '\n\t'
	SESroot.tail = '\n'
	
	SESwindow = ET.SubElement( SESroot, 'window', {
			'x'     : '0',
			'y'     : '0',
			'width' : '640',
			'height': '480',
			'role'  : window.get_role()
			}
		)
	
	SESwindow.text = '\n\t\t'
	SESwindow.tail = '\n'
	
	# The Tab List
	TabList = window.get_tabs()
	
	for aTab in TabList:
		SEStab = ET.SubElement( SESwindow, 'embed', { 'url':aTab.get_address(), 'title':aTab.get_title() } )
		SEStab.tail = '\n\t\t'
	
	SEStab.tail = '\n\t'
	
	stream = ET.tostring( SESroot, encoding="UTF-8")
	
	f = open( SESfile, 'w')
	f.write( stream )
	f.close()



def _load_tab_session( action, window ):
	
	# Ask for File
	SESfile = _show_dialog( window, 'LOAD' )
	
	# Returns None if dialog canceled
	if not SESfile:
		return
	
	SESstream = open( SESfile, 'r')
	
	for event, elem in ET.iterparse( SESstream ):
		if elem.tag == 'embed':
			
			tab = window.get_active_tab()
			
			if tab.get_property('address'):
				epiphany.ephy_shell_get_default().new_tab(
					window,
					tab,
					elem.attrib['url'],
					epiphany.NEW_TAB_IN_EXISTING_WINDOW | epiphany.NEW_TAB_APPEND_LAST | epiphany.NEW_TAB_OPEN_PAGE
					)
			else:
				window.load_url( elem.attrib['url'] )
			
			elem.clear()
	
	SESstream.close()



# This is to pass to gtk.ActionGroup.add_actions()
_actions = [
		('SaveTabSession', None, 'Save Tab Session', None, None, _save_tab_session),
		('LoadTabSession', None, 'Load Tab Session', None, None, _load_tab_session)
		]


# These implement the EphyExtension interface.

def attach_window(window):
	ui_manager = window.get_ui_manager()
	group = gtk.ActionGroup('TabSessionMgmt')
	group.add_actions(_actions, window)
	ui_manager.insert_action_group(group, -1)
	ui_id = ui_manager.add_ui_from_string(_ui_str)

	window._TabSessionMgmt_data = (group, ui_id)


def detach_window(window):
	group, ui_id = window._TabSessionMgmt_data
	del window._TabSessionMgmt_data

	ui_manager = window.get_ui_manager()
	ui_manager.remove_ui(ui_id)
	ui_manager.remove_action_group(group)
	ui_manager.ensure_update()

